package gs.smolban.db

import cats.data.EitherT
import gs.smolban.model.Ticket

trait TicketDb[F[_]]:
  /** Create a new [[Ticket]] in the database.
    *
    * @param ticket
    *   The [[Ticket]] to store.
    * @return
    *   The [[Ticket]] that was created, or an error.
    */
  def createTicket(ticket: Ticket): EitherT[F, DbError, Ticket]

  /** Read the specified [[Ticket]] from the database.
    *
    * @param ref
    *   The reference to the [[Ticket]] to retrieve.
    * @return
    *   The value of the [[Ticket]], or `None` if no such [[Ticket]] exists.
    */
  def readTicket(ref: Ticket.Reference): F[Option[Ticket]]

  /** Given a reference to a [[Ticket]] and some new value for that [[Ticket]],
    * update the [[Ticket]] in the database.
    *
    * @param ref
    *   The refrence to the [[Ticket]] to update.
    * @param newValue
    *   The new values with which to overwrite the [[Ticket]].
    * @return
    *   The modified [[Ticket]] if the operation succeeded, or `None` if the
    *   given reference does not exist.
    */
  def updateTicket(
    ref: Ticket.Reference,
    newValue: Ticket
  ): F[Option[Ticket]]

  /** Delete the specified [[Ticket]].
    *
    * @param ref
    *   The reference to the [[Ticket]] to delete.
    * @return
    *   `true` if the operation deleted a [[Ticket]], `false` otherwise.
    */
  def deleteTicket(ref: Ticket.Reference): F[Boolean]
